import middy, { MiddlewareObj } from '@middy/core';
import * as aws_lambda from 'aws-lambda';
import { APIGatewayProxyEvent, APIGatewayProxyResult, Context, ScheduledEvent } from 'aws-lambda';
import * as Joi from 'joi';
import { ObjectSchema } from 'joi';
import * as type_fest from 'type-fest';

declare class HttpError extends Error {
    name: string;
    statusCode: number;
    constructor(messageOrError: string | Error, statusCode: number);
}

declare class ServiceError extends Error {
    name: string;
    code: number;
    statusCode: number;
    constructor(messageOrError: string | Error, code?: number, statusCode?: number);
}

declare class BadRequestError extends HttpError {
    name: string;
    constructor(message?: string);
}

declare class ForbiddenError extends HttpError {
    name: string;
    constructor(message?: string);
}

declare class NotFoundError extends HttpError {
    name: string;
    constructor(message?: string);
}

type HttpErrorHandlerOptions = {
    defaultMessage?: string;
    defaultStatusCode?: number;
    logger?(message: string): void;
};
declare const httpErrorHandler: (opts?: HttpErrorHandlerOptions) => MiddlewareObj<APIGatewayProxyEvent, APIGatewayProxyResult>;

/**
 * ValidatorOptions
 */
type ValidatorOptions = {
    eventSchema?: Joi.ObjectSchema;
    logger?(message: string): void;
};
/**
 * Validate Lambda events with a Joi schema.
 * @param options - Options for the validator.
 * @returns Middleware to perform validation against a Joi schema.
 */
declare const validator: (options: ValidatorOptions) => MiddlewareObj;

type APIGatewayHandlerFn = (event: APIGatewayProxyEvent, context: Context) => Promise<APIGatewayProxyResult>;
type ScheduledHandlerFn = (event: ScheduledEvent, context: Context) => Promise<void>;
type MiddyfyOptions<THandler> = {
    handler: THandler;
    logger?: (message: string) => void;
};
type APIGatewayMiddyfyOptions = MiddyfyOptions<APIGatewayHandlerFn> & {
    eventSchema?: ObjectSchema;
    defaultErrorMessage?: string;
    defaultErrorStatusCode?: number;
};
type ScheduledMiddyfyOptions = MiddyfyOptions<ScheduledHandlerFn> & {
    eventSchema?: ObjectSchema;
};
declare const middyfyAPIGateway: (options: APIGatewayMiddyfyOptions) => middy.MiddyfiedHandler<APIGatewayProxyEvent & Omit<APIGatewayProxyEvent, "body"> & {
    body: type_fest.JsonValue;
} & {
    multiValueQueryStringParameters: aws_lambda.APIGatewayProxyEventMultiValueQueryStringParameters;
    pathParameters: aws_lambda.APIGatewayProxyEventPathParameters;
    queryStringParameters: aws_lambda.APIGatewayProxyEventQueryStringParameters;
}, APIGatewayProxyResult, Error, Context>;
declare const middyfyScheduled: (options: ScheduledMiddyfyOptions) => middy.MiddyfiedHandler<ScheduledEvent<any>, void, Error, Context>;

export { type APIGatewayHandlerFn, type APIGatewayMiddyfyOptions, BadRequestError, ForbiddenError, HttpError, type HttpErrorHandlerOptions, type MiddyfyOptions, NotFoundError, type ScheduledHandlerFn, type ScheduledMiddyfyOptions, ServiceError, type ValidatorOptions, httpErrorHandler, middyfyAPIGateway, middyfyScheduled, validator };
